﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using AuthLiteTokenProfileManager.Models;
using AuthLiteTokenProfileManager.Code;
using AuthLiteTokenProfileManager.Code.Constants;
using System.Configuration;
using System.ComponentModel;
using AuthLiteTokenProfileManager.Code.Helpers;
using log4net;
using TPMCommon;

namespace AuthLiteTokenProfileManager.Controllers
{
	public class HomeController : BaseController
	{
        // ILog can be declared in BaseController but then all debug messages output that scope instead of the inherited class which is helpful for debugging
        private static readonly ILog Log = log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

		public ActionResult Index()
		{
            if (SettingsHelper.FreeDomainSelection)
            {
                FreeDomainSelectionFormModel m = new FreeDomainSelectionFormModel();
                Log.Info("Return Free entry index view.");
                return View("FreeEntryIndex", m);
            }
            else
            {
                DomainSelectionFormModel m = new DomainSelectionFormModel();
                m.DomainOptions = GetDomainOptions();
                //m.Domain = m.DomainOptions.First().Value;
                Log.Info("Return Home Index view.");
                return View("Index", m);
            }
		}
		
		/// <summary>
		/// Handle form submission from portal landing page
		/// </summary>
		/// <param name="m"></param>
		/// <returns></returns>
		[HttpPost]
		[ValidateAntiForgeryToken]							
		public ActionResult Index(DomainSelectionFormModel m)
		{

            Log.Info("Home Form submitted. ");
			m.DomainOptions = GetDomainOptions();

			if (!ModelState.IsValid)
			{
				// Not valid, return to the form with user-entered values so they can resubmit
                Log.Info("Home form submitted model state is invalid. Returning to current view. ");
				return View(m);								
			}

            if(IdentityHelper.Instance.CheckAccess(m.Domain, m.UserName) != Collective.AuthLite.AccessControl.AccessCheckStatus.AccessGranted)
            {
                string msg = "This user is not permitted to use the portal, please try again.";
                ModelState.AddModelError("UserNotAllowed", msg);
                Log.Info("UserNotAllowed. " + msg);
                return View(m);
            }

            // don't even set these until after the above check, so that an unpermitted user can never sneak a valid session.
			Session[Constants.DOMAIN] = m.Domain;
			Session[Constants.USERNAME] = m.UserName;

            Log.Info("Form submission success. Domain and username set in session. Redirecting to Login Index Action. ");
            var res = RedirectToAction("Index", "Login"); // Real redirect								
            return res;
		}
		[HttpPost]
		[ValidateAntiForgeryToken]							
		public ActionResult FreeEntryIndex(FreeDomainSelectionFormModel m)
		{

            Log.Info("Home Form submitted. ");
            string[] du = m.DomainAndUserName.Split('\\');
			if (!ModelState.IsValid)
			{
				// Not valid, return to the form with user-entered values so they can resubmit
                Log.Info("Home form submitted model state is invalid. Returning to current view. ");
				return View(m);								
			}

            if(IdentityHelper.Instance.CheckAccess(du[0], du[1]) != Collective.AuthLite.AccessControl.AccessCheckStatus.AccessGranted)
            {
                string msg = "This user is not permitted to use the portal, please try again.";
                ModelState.AddModelError("UserNotAllowed", msg);
                Log.Info("UserNotAllowed. " + msg);
                return View(m);
            }

            // don't even set these until after the above check, so that an unpermitted user can never sneak a valid session.
			Session[Constants.DOMAIN] = du[0];
			Session[Constants.USERNAME] = du[1];

            Log.Info("Form submission success. Domain and username set in session. Redirecting to Login Index Action. ");
            var res = RedirectToAction("Index", "Login"); // Real redirect								
            return res;
		}		
		public ActionResult SessionExpired()
		{
			TempData["sessionExpired"] = true;
            Log.Info("Session Expired. Redirecting to Home Index Action");
			return RedirectToAction("Index");
		}

        public ActionResult NowAuthLiteUser()
        {
            // I *should* be able to do this, but for some reason it ruins the TempData
            // (I did have it showing ok when I returned a View instead of doing a Redirect,
            // but then the Submit button on the form wouldn't go to the right action even after I 
            // explicitly made BeginForm("Index")
            // Anyway, I just made the caller Dispose the WindowsIdentity, and that's pretty well
            // good enough to be safe.
            //SessionHelper.End();

            TempData["success"] = "You are now an AuthLite user. Please log in again using 2-factor.";
            Log.Info("NowAuthLiteUser. ");
            return RedirectToAction("Index");
        }

		/// <summary>
		/// Populate Domain dropdown options
		/// </summary>
		/// <returns></returns>
		private SelectList GetDomainOptions()
		{
            string domains = ConfigurationManager.AppSettings["Domains"].ToString();
            List<SelectListItem> list = new List<SelectListItem>();
            foreach(string s in domains.Split('|'))
            {
                SelectListItem item = new SelectListItem();
                item.Text = s;
                item.Value = s;
                list.Add(item);
            }
            SelectList options = new SelectList(list);
			return options;
		}

		
		// Getting the form actionresults to maintain the url is difficult when using custom named methods (we can probably route them, but that's a bunch more work for minimal benefit)
		// http://stackoverflow.com/questions/752922/asp-mvc-redirect-without-changing-urlrouting
		// We can probably route them (link below), but that's a lot of work rather than just making different controllers that naturally match each page name
		// http://stackoverflow.com/questions/27626391/how-can-i-use-asp-net-mvc-routing-to-call-a-different-actionresult-method-and-di
		/*
		public ActionResult RenderDomainSelectionForm()
		{
			DomainSelectionFormModel m = new DomainSelectionFormModel();
			m.DomainOptions = GetDomainOptions();
			return PartialView("DomainSelectionForm", m);
		}
				
		public ActionResult RenderAssignForm()
		{
			AssignFormModel m = new AssignFormModel();
			return PartialView("AssignForm", m);
		}

		[HttpPost]
		[ValidateAntiForgeryToken]
		public ActionResult HandleAssignForm(AssignFormModel m)
		{
			return View(m);
		}
		*/

	}
}